Scriptname SPEBestFriendPetScript extends ReferenceAlias
{Manages your companion pet's many functions.}

Actor Property PlayerRef Auto
Actor CurrentPet
ReferenceAlias Property TalkingAlias Auto
ObjectReference DialogueActivator

;state variables
GlobalVariable Property SPEBF_Selling Auto
GlobalVariable Property SPEBF_Storing Auto
GlobalVariable Property SPEBF_PetType Auto
GlobalVariable Property SPEBF_InDialogue Auto
GlobalVariable Property SPEHardcoreMode Auto
float Property CustomPetDamageMult = 0.6 Auto
int PlayerSpeech
int PetIndex
bool NeedToReset = false
bool InitializeSale = false
bool CheckPlayerDistance = false
bool BleedingOut = false
float HardcoreDivisor = 1.0

Function PetInit()
	CurrentPet = GetActorRef()
	DialogueActivator = TalkingAlias.GetRef()
	CurrentPet.SetAV("Lockpicking", 0) ;Bethesda really has a thing about animals and lockpicking, and who am I to argue?
	CurrentPet.SetNoBleedoutRecovery(false)
	CurrentPet.AllowBleedoutDialogue(true)
	CurrentPet.SetRelationshipRank(PlayerRef, 3)
	CurrentPet.SetPlayerTeammate(abCanDoFavor = false)
	;CurrentPet.BlockActivation() ;use the talking activator instead
	PetIndex = SPEBF_PetType.GetValue() as int
	SetDamage()
	RegisterForMenu("Dialogue Menu")
	RegisterForModEvent("SPE_HardcoreModeOn", "OnHardcoreSwitch")
	RegisterForModEvent("SPE_HardcoreModeOff", "OnHardcoreSwitch")
	RegisterForUpdateGameTime(1)
EndFunction

Function OnHardcoreSwitch(string eventName, string strArg, float numArg, Form sender)
	SetDamage()
EndFunction

Event OnMenuClose(String MenuName)
	DialogueActivator.Disable()
	SPEBF_InDialogue.SetValue(0)
	CurrentPet.ClearLookAt()
	CurrentPet.EvaluatePackage()
EndEvent

Function SetDamage()
	PlayerSpeech = PlayerRef.GetAV("Speechcraft") as int
	If SPEHardcoreMode.GetValue() == 1
		HardcoreDivisor = 2.5
	Else
		HardcoreDivisor = 1.0
	EndIf
	If PetIndex == 0 || PetIndex == 5 || PetIndex == 8 ;mammoth, horker, wolf - low attack power
		CurrentPet.SetAV("UnarmedDamage", PlayerSpeech * 0.5 / HardcoreDivisor)
	ElseIf PetIndex == 1 ;bear - good attacker, but slow
		CurrentPet.SetAV("UnarmedDamage", PlayerSpeech * 0.8 / HardcoreDivisor)
	ElseIf PetIndex == 7 || PetIndex == 6 ;cat, mudcrab - good attackers
		CurrentPet.SetAV("UnarmedDamage", PlayerSpeech * 0.7 / HardcoreDivisor)
	ElseIf PetIndex <= 8 ;everything else attacks about the same
		CurrentPet.SetAV("UnarmedDamage", PlayerSpeech * 0.6 / HardcoreDivisor)
	Else ;pet added by a mod
		CurrentPet.SetAV("UnarmedDamage", PlayerSpeech * CustomPetDamageMult / HardcoreDivisor)
	EndIf
EndFunction

Event OnActivate(ObjectReference akActionRef)
	If akActionRef == PlayerRef && PlayerRef.IsInCombat() == 0 ;no dialogue in combat
		;Debug.Trace("Starting dialogue.")
		DialogueActivator.MoveTo(CurrentPet)
		DialogueActivator.Enable()
		CurrentPet.SetLookAt(PlayerRef, true)
		DialogueActivator.Activate(akActionRef)
		SPEBF_InDialogue.SetValue(1)
		CurrentPet.EvaluatePackage()
	EndIf
EndEvent

Event OnEnterBleedout()
	BleedingOut = true
	CurrentPet.PushActorAway(CurrentPet, 1)
	CurrentPet.SetAV("Paralysis", 1)
	RegisterForSingleUpdate(10)
EndEvent

Event OnUpdate()
	If InitializeSale == true
		InitializeSale = false
		Float PlayerSkill = Game.GetPlayer().GetAV("Speechcraft")
		PriceModifier = (Game.GetGameSettingFloat("fBarterMax") * (100 - PlayerSkill) + Game.GetGameSettingFloat("fBarterMin") * PlayerSkill) / 100
		SaleTotal = 0
		SPEBF_Selling.SetValue(1)
		lastx = CurrentPet.X
		lasty = CurrentPet.Y
		lastz = CurrentPet.Z
		LastLocation = CurrentPet.GetCurrentLocation()
		If NeedToReset == true && GameDaysPassed.GetValue() >= DayToReset
			ResetChests()
		EndIf
		If NeedToReset == false 
			NeedToReset = true
			DayToReset = GameDaysPassed.GetValue() + 2
			ChestsToReset = new ObjectReference[100] ;ludicrously huge, limit should never be hit
			ChestIndex = 0
		EndIf
	EndIf
	If BleedingOut == true
		BleedingOut = false
		CurrentPet.SetAV("Paralysis", 0)
		CurrentPet.StopCombat()
	EndIf
	If CheckPlayerDistance == true
		CheckPlayerDistance = false
		If CanCheckDistance(PlayerRef) == false || CurrentPet.GetDistance(PlayerRef) > MaxDistance
			CurrentPet.MoveToPackageLocation()
		EndIf
	EndIf
EndEvent

Worldspace Property Tamriel Auto
Location Property WinterholdCollegeLocation Auto
ObjectReference Property WinterholdFastTravelMarker Auto

Event OnLoad()
	If CurrentPet.IsInLocation(WinterholdCollegeLocation)
		If CurrentPet.GetWorldspace() == Tamriel && CurrentPet.GetDistance(WinterholdFastTravelMarker) < 500
			If (PlayerRef.Z - 20) < WinterholdFastTravelMarker.Z ;player fell
				CurrentPet.MoveTo(WinterholdFastTravelMarker, -150, -150)
				PlayerRef.MoveTo(WinterholdFastTravelMarker)
			EndIf
		EndIf
	EndIf
EndEvent

;/--------------------------------------------------------------------------
|																			|
|									TRAVEL									|
|																			|
 ---------------------------------------------------------------------------

This section contains generic code related to pet travel and movement.

/;

GlobalVariable Property SPEBF_Travel Auto
Message Property SPEBF_StoreMessage Auto
Message Property SPEBF_NoSaleMessage Auto
Message Property SPEBF_SaleMessage Auto
Package Property SPEBF_WanderPackage Auto
float Property MaxDistance = 2048.0 Auto
{The pet will teleport to the player if it gets more than this far away for too long.}
float lastx = 0.0
float lasty = 0.0
float lastz = 0.0
Location LastLocation = NONE


Event OnUpdateGameTime()

	;reset chests if needed
	If NeedToReset && GameDaysPassed.GetValue() >= DayToReset
		ResetChests()
	EndIf
	
	;teleport pet to the player if it gets lost or separated
	If IsPetStuck() == true
		CurrentPet.MoveToPackageLocation()
	EndIf
	
	;update pet damage if needed
	If PlayerSpeech != PlayerRef.GetAV("Speechcraft") as int
		SetDamage()
	EndIf

EndEvent

bool Function IsPetStuck()

	;determine whether or not the pet has moved
	Location CurrentLocation = CurrentPet.GetCurrentLocation()
	float xdist = Math.abs(CurrentPet.X - lastx)
	float ydist = Math.abs(CurrentPet.Y - lasty)
	float zdist = Math.abs(CurrentPet.Z - lastz)
	bool Moved = true
	If LastLocation == CurrentLocation && (xdist + ydist + zdist) < 128.0 ;pet has barely moved
		Moved = false
	EndIf
	
	;update for next time
	LastLocation = CurrentLocation
	lastx = CurrentPet.X
	lasty = CurrentPet.Y
	lastz = CurrentPet.Z
	
	;check to see if the pet should have moved and report
	If CurrentPet.GetAV("WaitingForPlayer") == 1
		If SPEBF_Travel.GetValue() == 0 ;just waiting, skip check
			return false
		ElseIf CurrentPet.GetCurrentPackage() == SPEBF_WanderPackage ;pet made it
			return false
		ElseIf Moved == false ;pet didn't make it and isn't moving
			return true
		Else
			return false
		EndIf
	ElseIf SPEBF_Selling.GetValue() == 1 ;pet is seeking merchant
		Actor Merchant = MerchantAlias.GetActorRef()
		If CanCheckDistance(Merchant) && CurrentPet.GetDistance(Merchant) <= 256.0
			return false
		ElseIf Moved == false
			return true
		Else
			return false
		EndIf
	Else ;pet is seeking player
		If CanCheckDistance(PlayerRef) && CurrentPet.GetDistance(PlayerRef) <= MaxDistance
			return false
		ElseIf Moved == false
			return true
		Else ;pet is in a different cell or distant, but still moving
			CheckPlayerDistance = true
			RegisterForSingleUpdate(20) ;give the pet 20 seconds to catch up or be teleported
			return false
		EndIf
	EndIf

EndFunction

bool Function CanCheckDistance(ObjectReference Target)
{Checks to make sure the pet and target are in the same space for a GetDistance check}
	If CurrentPet.IsInInterior()
		If CurrentPet.GetParentCell() == Target.GetParentCell()
			return true
		Else
			return false
		EndIf
	ElseIf CurrentPet.GetWorldspace() == Target.GetWorldSpace()
		return true
	Else
		return false
	EndIf
EndFunction

Function PetReturned()
	If SPEBF_Selling.GetValue() == 2
		SPEBF_Selling.SetValue(0)
		If SaleTotal > 0
			SPEBF_SaleMessage.Show(SaleTotal)
		Else
			SPEBF_NoSaleMessage.Show()
		EndIf
	ElseIf SPEBF_Storing.GetValue() == 2
		SPEBF_Storing.SetValue(0)
		SPEBF_StoreMessage.Show()
	EndIf
	;Debug.Trace("Pet has returned. Success!")
EndFunction

;/--------------------------------------------------------------------------
|																			|
|								SELLING										|
|																			|
 ---------------------------------------------------------------------------

This section contains code related to selling items.

/;

Keyword Property LocTypeHabitationHasInn Auto
LocationAlias Property LastTownAlias Auto
Quest Property SPEBF_SellQuest Auto
GlobalVariable Property SPEBF_SimpleSelling Auto
GlobalVariable Property SPEBF_LowestPrice Auto
GlobalVariable Property GameDaysPassed Auto
ReferenceAlias Property MerchantAlias Auto
ReferenceAlias Property MerchantChestAlias Auto
ReferenceAlias Property MerchantChestAlias02 Auto
ReferenceAlias Property MerchantAlias02 Auto
FormList Property SPEBF_UsedMerchantList Auto
FormList Property SPEBF_UsedChestList Auto
Message Property SPEBF_SellFailMessage Auto
MiscObject Property Gold001 Auto
float DayToReset
ObjectReference[] ChestsToReset
int ChestIndex = 0
int GoldValue
int SaleTotal
float PriceModifier

Event OnLocationChange(Location akOldLoc, Location akNewLoc)
	If akNewLoc != NONE && akNewLoc.HasKeyword(LocTypeHabitationHasInn)
		;Debug.Trace("Found new town. Updating 'last town' alias.")
		LastTownAlias.ForceLocationTo(akNewLoc)
	EndIf
EndEvent

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	int value = 0
	If akBaseItem != Gold001
		value = akBaseItem.GetGoldValue()
	EndIf
	If value > 0
		GoldValue += value*aiItemCount
		;Debug.Trace("New gold value: " + GoldValue)
	EndIf
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	int value = 0
	If akBaseItem != Gold001
		value = akBaseItem.GetGoldValue()
	EndIf
	If value > 0
		GoldValue -= value*aiItemCount
	EndIf
EndEvent

Function StartSelling()
	If LastTownAlias.GetLocation() == NONE
		SPEBF_SellFailMessage.Show()
	Else
		If SPEBF_SellQuest.Start() == true
			If SPEBF_SimpleSelling.GetValue() == 0 && MerchantChestAlias.GetRef() == NONE && MerchantChestAlias02.GetRef() == NONE
				;Debug.Trace("Shopkeeper found, but no available merchant chests.")
				SPEBF_SellQuest.Stop()
			Else
				InitializeSale = true
				RegisterForSingleUpdate(1) ;let the dialogue end quickly
			EndIf
		EndIf
	EndIf
EndFunction

Function Sell()
	Actor CurrentMerchant = MerchantAlias.GetActorRef()
	ObjectReference CurrentChest = MerchantChestAlias.GetRef()
	If CurrentChest == NONE
		CurrentChest = MerchantChestAlias02.GetRef() ;can't find the real merchant chest, so just use any valid chest
		CurrentMerchant = MerchantAlias02.GetActorRef() ;try to 'use' the merchant the new chest belongs to
	EndIf
	If SPEBF_SimpleSelling.GetValue() == 1
		int ItemCount
		int TotalCost
		int SellValue
		Form CurrentItem
		int ItemIndex = CurrentPet.GetNumItems()
		While ItemIndex > 0
			ItemIndex -= 1
			CurrentItem = CurrentPet.GetNthForm(ItemIndex)
			If CurrentItem != Gold001 ;don't "sell" gold
				SellValue = (CurrentItem.GetGoldValue() as Float / PriceModifier) as int
				If SellValue > 0 ;don't sell items for nothing
					ItemCount = CurrentPet.GetItemCount(CurrentItem)
					TotalCost = ItemCount * SellValue
					CurrentPet.RemoveItem(CurrentItem, ItemCount, akOtherContainer = CurrentChest)
					CurrentPet.AddItem(Gold001, TotalCost)
					SaleTotal += TotalCost
				EndIf
			EndIf
		EndWhile
		StopSelling()
	Else
		int MerchantGold = CurrentChest.GetItemCount(Gold001)
		int ItemCount = 0
		Form CurrentItem
		int CurrentSellPrice
		int ItemIndex = CurrentPet.GetNumItems()
		int LowPrice = 0
		bool sellable = false
		While ItemIndex > 0 ;iterate through all items, selling as much as possible
			ItemIndex -= 1
			CurrentItem = CurrentPet.GetNthForm(ItemIndex)
			;Debug.Trace("Checking " + CurrentItem.GetName())
			If CurrentItem != Gold001 ;don't "sell" gold
				ItemCount = CurrentPet.GetItemCount(CurrentItem)
				CurrentSellPrice = (CurrentItem.GetGoldValue() as Float / PriceModifier) as int
				If CurrentSellPrice > 0
					If ItemCount == 1 && MerchantGold >= CurrentSellPrice
						CurrentPet.RemoveItem(CurrentItem, akOtherContainer = CurrentChest)
						CurrentChest.RemoveItem(Gold001, CurrentSellPrice, akOtherContainer = CurrentPet)
						MerchantGold -= CurrentSellPrice
						SaleTotal += CurrentSellPrice
						;Debug.Trace("Sold one item for " + CurrentSellPrice + " gold. Merchant has " + MerchantGold + " left.")
					Else
						int PurchaseLimit = MerchantGold / CurrentSellPrice
						If PurchaseLimit > 0
							int NumberSold = ItemCount
							If ItemCount > PurchaseLimit ;can't sell all
								NumberSold = PurchaseLimit
								sellable = true
								;Debug.Trace("Can't sell all. " + (ItemCount - PurchaseLimit) + " items will remain.")
								If LowPrice > CurrentSellPrice || LowPrice == 0
									LowPrice = CurrentSellPrice
								EndIf
							EndIf
							int TotalCost = NumberSold * CurrentSellPrice
							CurrentPet.RemoveItem(CurrentItem, NumberSold, akOtherContainer = CurrentChest)
							CurrentChest.RemoveItem(Gold001, TotalCost, akOtherContainer = CurrentPet)
							MerchantGold -= TotalCost
							SaleTotal += TotalCost
							;Debug.Trace("Sold " + NumberSold + " items for " + TotalCost + " gold. Merchant has " + MerchantGold + " left.")
						EndIf
					EndIf
				EndIf
			EndIf
		EndWhile
		SPEBF_LowestPrice.SetValue(LowPrice)
		;Debug.Trace("Lowest price: " + LowPrice)
		If CurrentMerchant != NONE ;make sure a merchant was actually found
			SPEBF_UsedMerchantList.AddForm(CurrentMerchant.GetActorBase()) ;don't use the same merchant twice
		EndIf
		SPEBF_UsedChestList.AddForm(CurrentChest.GetBaseObject()) ;don't use the same chest twice
		If ChestsToReset.Find(CurrentChest) < 0
			ChestsToReset[ChestIndex] = CurrentChest
			ChestIndex += 1
			;Debug.Trace("Noted " + ChestIndex + " chests to reset.")
		EndIf
		;Debug.Trace("Visited " + SPEBF_UsedMerchantList.GetSize() + " merchants and used " + SPEBF_UsedChestList.GetSize() + " chests.")
		If GoldValue == 0 || sellable == false
			;Debug.Trace("Stopped selling with " + GoldValue + " gold value remaining.")
			StopSelling()
		Else
			;Debug.Trace("Have " + GoldValue + " left to sell. Finding another merchant..." )
			SPEBF_SellQuest.Reset()
			SPEBF_SellQuest.Stop()
			If SPEBF_SellQuest.Start() == true ;another merchant found, keep going
				If MerchantChestAlias.GetRef() == NONE && MerchantChestAlias02.GetRef() == NONE
					;Debug.Trace("Found merchant but no chest. Aborting.")
					StopSelling()
				EndIf
			Else
				StopSelling()
			EndIf
		EndIf
	EndIf
EndFunction

Function StopSelling()
	;Debug.Trace("Selling finished. Returning to player.")
	SPEBF_SellQuest.Stop()
	SPEBF_Selling.SetValue(2) ;tell the pet to come back
	SPEBF_LowestPrice.SetValue(0) ;make sure the first merchant check doesn't fail from too little gold
	SPEBF_UsedMerchantList.Revert()
	SPEBF_UsedChestList.Revert()
EndFunction


Function ResetChests()
	;Debug.Trace("Resetting " + ChestIndex + " chests.")
	int i = ChestIndex
	While i > 0
		i -= 1
		ChestsToReset[i].Reset()
	EndWhile
	NeedToReset = false
EndFunction

;/--------------------------------------------------------------------------
|																			|
|							STORE FUNCTIONS									|
|																			|
 ---------------------------------------------------------------------------

This section contains events and functions related to storing items at home.

/;

Quest Property SPEBF_StoreQuest Auto
Message Property SPEBF_StoreFailMessage Auto
ReferenceAlias Property ChestAlias01 Auto
ReferenceAlias Property ChestAlias02 Auto
ReferenceAlias Property ChestAlias03 Auto
ReferenceAlias Property ChestAlias04 Auto
ObjectReference TargetChest

Function StartStoring()
	If SPEBF_StoreQuest.Start() == true
		TargetChest = ChestAlias01.GetRef()
		If TargetChest != NONE
			SPEBF_Storing.SetValue(1)
		Else
			TargetChest = ChestAlias02.GetRef()
			If TargetChest != NONE
				SPEBF_Storing.SetValue(2)
			Else
				TargetChest = ChestAlias03.GetRef()
				If TargetChest != NONE
					SPEBF_Storing.SetValue(3)
				Else
					TargetChest = ChestAlias01.GetRef()
					If TargetChest != NONE
						SPEBF_Storing.SetValue(4)
					Else
						SPEBF_StoreQuest.Stop() ;no chests found
						SPEBF_StoreFailMessage.Show()
					EndIf
				EndIf
			EndIf
		EndIf
	EndIf
	;CurrentPet.EvaluatePackage()
EndFunction

Function Store()
	CurrentPet.RemoveAllItems(TargetChest)
	SPEBF_Storing.SetValue(2)
	SPEBF_StoreQuest.Stop()
EndFunction