Scriptname SPEEtherealAmmoScript extends SPEPlayerOnlyAbility
{Script to manage automatic bound arrows/bolts without requiring a spell cast.}

Perk Property SPESpiritBinding Auto
Perk Property SPEMysticBinding Auto
Perk Property SPEOblivionBinding Auto
Keyword Property WeapTypeBow Auto
Keyword Property VendorItemArrow Auto
Keyword Property WeapTypeBoundArrow Auto
Keyword Property SPEEtherealAmmo Auto
GlobalVariable Property SPEEtherealAmmoMode Auto
GlobalVariable Property SPEHardcoreMode Auto
FormList Property SPEEtherealArrowList Auto
FormList Property SPEEtherealBoltList Auto
int Property MaxAmmo = 100 Auto
int ammoCount
Ammo CurrentArrow
Ammo CurrentBolt

Function Initialize()
	RegisterForModEvent("SPE_EtherealAmmoRegister", "RegisterAmmoKey")
	RegisterForModEvent("SPE_EtherealAmmoClear", "ClearAmmoKey")
	RegisterForModEvent("SPE_EtherealAmmoSwitch", "ModeSwitch")
	RegisterForModEvent("SPE_HardcoreModeOn", "HardcoreSwitch")
	RegisterForModEvent("SPE_HardcoreModeOff", "HardcoreSwitch")
	SetAmmo()
	If SPEEtherealAmmoMode.GetValue() == 1
		GoToState("AutomaticMode")
	EndIf
EndFunction

Function SetAmmo()
	int index
	If SPEHardcoreMode.GetValue() == 1 || PlayerRef.HasPerk(SPESpiritBinding) == 0
		index = 0
	ElseIf PlayerRef.HasPerk(SPEOblivionBinding)
		index = 6
	ElseIf PlayerRef.HasPerk(SPEMysticBinding)
		index = 4
	Else
		index = 2
	EndIf
	If SPEEtherealAmmoMode.GetValue() == 2
		index += 1
	EndIf
	Ammo NewArrow = SPEEtherealArrowList.GetAt(index) as Ammo
	Ammo NewBolt = SPEEtherealBoltList.GetAt(index) as Ammo
	If CurrentArrow != NewArrow || CurrentBolt != NewBolt ;ammo has changed
		RemoveAllAmmo()
		CurrentArrow = NewArrow
		CurrentBolt = NewBolt
	EndIf
EndFunction

Function HardcoreSwitch(string eventName, string strArg, float numArg, Form sender)
	SetAmmo()
	RegisterForSingleUpdate(0)
EndFunction

Function ModeSwitch(string eventName, string strArg, float numArg, Form sender)
	UnregisterForAllKeys()
	SetAmmo()
	If SPEEtherealAmmoMode.GetValue() == 1 ;auto mode
		GoToState("AutomaticMode")
		RegisterForSingleUpdate(0)
	Else
		GoToState("") ;ignore automatic functions
	EndIf
EndFunction

; MANUAL MODE ----------------------------------------------------------------------

Function RegisterAmmoKey(string eventName, string strArg, float numArg, Form sender)
	RegisterForKey(numArg as int)
EndFunction

Function ClearAmmoKey(string eventName, string strArg, float numArg, Form sender)
	UnregisterForAllKeys()
EndFunction

Event OnKeyDown(int KeyCode)
	If SPEEtherealAmmoMode.GetValue() == 2 ;double check to make sure
		SetAmmo() ;check for new ammo
		bool ammoRefilled = false
		ammoCount = PlayerRef.GetItemCount(CurrentArrow)
		If ammoCount < MaxAmmo
			PlayerRef.AddItem(CurrentArrow, MaxAmmo - ammoCount, true)
			If PlayerRef.IsEquipped(CurrentArrow)
				PlayerRef.UnequipItem(CurrentArrow, false, true)
				PlayerRef.EquipItem(CurrentArrow, false, true)
			EndIf
			ammoRefilled = true
		EndIf
		ammoCount = PlayerRef.GetItemCount(CurrentBolt)
		If ammoCount < MaxAmmo
			PlayerRef.AddItem(CurrentBolt, MaxAmmo - ammoCount, true)
			If PlayerRef.IsEquipped(CurrentBolt)
				PlayerRef.UnequipItem(CurrentBolt, false, true)
				PlayerRef.EquipItem(CurrentBolt, false, true)
			EndIf
			ammoRefilled = true
		EndIf
		If ammoRefilled
			If PlayerRef.GetAV("Magicka") > 0
				PlayerRef.DamageAV("Magicka", PlayerRef.GetAV("Magicka") / 2)
			EndIf
		EndIf
	EndIf
EndEvent

Function RemoveAmmo()
	PlayerRef.RemoveItem(CurrentArrow, PlayerRef.GetItemCount(CurrentArrow), true)	
	PlayerRef.RemoveItem(CurrentBolt, PlayerRef.GetItemCount(CurrentBolt), true)	
EndFunction

Function RemoveAllAmmo()
	int i = SPEEtherealArrowList.GetSize()
	While i > 0
		i -= 1
		PlayerRef.RemoveItem(SPEEtherealArrowList.GetAt(i), PlayerRef.GetItemCount(SPEEtherealArrowList.GetAt(i)), true)
		PlayerRef.RemoveItem(SPEEtherealBoltList.GetAt(i), PlayerRef.GetItemCount(SPEEtherealBoltList.GetAt(i)), true)
	EndWhile
EndFunction

; AUTOMATIC MODE -------------------------------------------------------------------

State AutomaticMode
	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
		If akBaseObject.HasKeyword(WeapTypeBow)
			RegisterForSingleUpdate(0)
		ElseIf akBaseObject.HasKeyword(VendorItemArrow)
			RemoveAmmo()
		EndIf
	EndEvent

	Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
		If akBaseObject.HasKeyword(WeapTypeBow) || akBaseObject.HasKeyword(VendorItemArrow)
			RegisterForSingleUpdate(0)
		EndIf
	EndEvent

	Event OnUpdate()
		If PlayerRef.GetEquippedItemType(1) == 7 && PlayerRef.WornHasKeyword(VendorItemArrow) == 0 ;arrows should be equipped
			If PlayerRef.WornHasKeyword(WeapTypeBoundArrow) && PlayerRef.WornHasKeyword(SPEEtherealAmmo) == 0 ;bound arrows equipped
				RemoveAmmo()
			Else
				SetAmmo() ;update ammo if needed
				ammoCount = PlayerRef.GetItemCount(CurrentArrow)
				If  ammoCount < MaxAmmo ;refill arrows
					PlayerRef.AddItem(CurrentArrow, MaxAmmo - ammoCount, true)
					PlayerRef.UnequipItem(CurrentArrow, false, true)
				EndIf
				PlayerRef.EquipItem(CurrentArrow, true, true)
				GoToState("ArrowsEquipped")
			EndIf
		ElseIf PlayerRef.GetEquippedItemType(1) == 12 && PlayerRef.WornHasKeyword(VendorItemArrow) == 0 ;bolts should be equipped
			SetAmmo() ;update ammo if needed
			ammoCount = PlayerRef.GetItemCount(CurrentBolt)
			If ammoCount < MaxAmmo ;refill bolts
				PlayerRef.AddItem(CurrentBolt, MaxAmmo - ammoCount, true)
				PlayerRef.UnequipItem(CurrentBolt, false, true)
				If PlayerRef.GetAV("Magicka") > 0
					PlayerRef.DamageAV("Magicka", PlayerRef.GetAV("Magicka") / 2)
				EndIf
			EndIf
			PlayerRef.EquipItem(CurrentBolt, true, true)
		Else ;get rid of ammo
			RemoveAmmo()
		EndIf
	EndEvent
EndState

State ArrowsEquipped
	Event OnPlayerBowShot(Weapon akWeapon, Ammo akAmmo, float afPower, bool abSunGazing)
		If PlayerRef.GetAv("Magicka") > 9
			PlayerRef.DamageAv("Magicka", 10)
		Else
			PlayerRef.DamageAv("Magicka", PlayerRef.GetAv("Magicka"))
		EndIf
		If PlayerRef.GetItemCount(CurrentArrow) == 1
			SetAmmo()
			PlayerRef.UnEquipItem(CurrentArrow, true, true)
			PlayerRef.AddItem(CurrentArrow, MaxAmmo - PlayerRef.GetItemCount(CurrentArrow), true)
			PlayerRef.EquipItem(CurrentArrow, true, true)
		EndIf
	EndEvent
	
	Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
		If akBaseObject.HasKeyword(VendorItemArrow)
			RemoveAmmo()
			GoToState("AutomaticMode")
			RegisterForSingleUpdate(0)
		EndIf
	EndEvent
	
	Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
		If PlayerRef.GetEquippedItemType(1) != 7
			RemoveAmmo()
			GoToState("AutomaticMode")
			RegisterForSingleUpdate(0)
		EndIf
	EndEvent
	
EndState