Scriptname SPEFaithScript extends SPEPlayerOnlyAbility
{Converts overhealing into temporary hit points.}

GlobalVariable Property SPEHardcoreMode Auto
Keyword Property MagicRestoreHealth Auto
Spell Property SPEPerkFaithFort01 Auto
Spell Property SPEPerkFaithFort02 Auto
Spell Property SPEPerkFaithFort03 Auto
Spell Property SPEPerkFaithFort04 Auto
Spell Property SPEPerkFaithFort05 Auto
Spell Property SPEPerkFaithFort06 Auto
Spell Property SPEPerkFaithFort07 Auto
Spell Property SPEPerkFaithFort08 Auto
Spell Property SPEPerkFaithFort09 Auto
Spell Property SPEPerkTemperanceShell Auto
Spell Property SPEPerkTemperanceFort01 Auto
Spell Property SPEPerkTemperanceFort02 Auto
Spell Property SPEPerkTemperanceFort03 Auto
Spell Property SPEPerkTemperanceFort04 Auto
Spell Property SPEPerkTemperanceFort05 Auto
Spell Property SPEPerkTemperanceFort06 Auto
Spell Property SPEPerkTemperanceFort07 Auto
Spell Property SPEPerkTemperanceFort08 Auto
Spell Property SPEPerkTemperanceFort09 Auto
Perk Property SPETemperance Auto
float CurrentHealth
Int MaxHealth
int FortifyAmount = 0
int FortifiedStamina = 0
bool UpdatingStamina = false
Spell SpellCast
MagicEffect CurrentEffect

Function Initialize()
	RegisterForModEvent("SPE_TemperanceClearBuffs", "ClearTemperanceBuffs")
	CurrentHealth = PlayerRef.GetAV("Health")
	If PlayerRef.GetAVPercentage("Health") == 1
		MaxHealth = CurrentHealth as int
	Else
		MaxHealth = PlayerRef.GetBaseAV("Health") as int ;possibly inaccurate, but it'll fix itself soon enough
	EndIf
EndFunction

Function ClearTemperanceBuffs(string eventName, string strArg, float numArg, Form sender)
	RemoveStaminaSpells()
EndFunction

Event OnHit(ObjectReference akAggressor, Form akSource, Projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
	CurrentHealth = PlayerRef.GetAV("Health")
	If FortifyAmount > 0 ;fortified, recalculate total fortify
		float BaseHealth = PlayerRef.GetBaseAV("Health")
		If (CurrentHealth < BaseHealth) && (MaxHealth > BaseHealth) ;clear buffs unless max health is damaged
			ClearBuffs()
		Else ;normal hit, or max health is damaged
			int DamageTaken = MaxHealth - CurrentHealth as int
			If DamageTaken > 0 ;temporary health should be removed
				UpdateFortify(DamageTaken)
			ElseIf DamageTaken < 0 ;hit points went up for some reason, probably another fortify effect
				MaxHealth = (CurrentHealth / PlayerRef.GetAVPercentage("Health")) as int ;estimate new max health on the fly
				DamageTaken = MaxHealth - CurrentHealth as int ;recalculate damage
				If DamageTaken > 0 ;hopefully we got it right this time, but if not, give up until the next hit
					UpdateFortify(DamageTaken)
				EndIf
			EndIf
		EndIf
	EndIf
EndEvent

Function UpdateFortify(int DamageTaken)
	int NewFortify = FortifyAmount - DamageTaken
	If NewFortify > 0
		PlayerRef.RestoreAV("Health", DamageTaken) ;heal before removing hit points or damage taken increases
		FortifyHealth(NewFortify)
	Else ;should only trigger if max health is damaged
		ClearBuffs()
	EndIf
EndFunction

Function ClearBuffs()
	PlayerRef.RestoreAV("Health", FortifyAmount)
	RemoveSpells()
	If PlayerRef.GetAVPercentage("Health") == 1
		MaxHealth = PlayerRef.GetAV("Health") as int
	Else
		MaxHealth -= FortifyAmount
	EndIf
	FortifyAmount = 0
EndFunction

Event OnUpdate()
	;by the time a spell is cast, it's too late to get the player's health, so track it while it regenerates
	CurrentHealth = PlayerRef.GetAV("Health")
	If PlayerRef.GetAVPercentage("Health") == 1
		MaxHealth = CurrentHealth as int ;re-update max health when we can
	EndIf
EndEvent

Event OnSpellCast(Form akSpell)
	If akSpell.HasKeyword(MagicRestoreHealth) && SpellCast == NONE ;don't overwrite mid-processing
		SpellCast = akSpell as Spell
		If CurrentEffect != NONE ;effect was applied to self
			CalculateFortify()
		Else
			Utility.Wait(0.1) ;try again shortly just in case of a timing error
			If CurrentEffect != NONE
				CalculateFortify()
			Else
				SpellCast = NONE ;nothing happened, so return to normal
			EndIf
		EndIf
	EndIf
EndEvent

Function CalculateFortify()
	int i = SpellCast.GetNumEffects()
	While i > 0
		i -= 1
		If SpellCast.GetNthEffectMagicEffect(i) == CurrentEffect
			;Debug.Trace("Current effect found.")
			float HealthAtCastTime = CurrentHealth ;CurrentHealth likes to change during this calculation
			;the following formula calculates final magnitude based on skill and fortify effects
			float TotalHealing = SpellCast.GetNthEffectMagnitude(i) * (PlayerRef.GetAV("Restoration") * 0.01 + 1) * (PlayerRef.GetAV("RestorationMod") * 0.01 + 1) * (PlayerRef.GetAV("RestorationPowerMod") * 0.01 + 1)
			float CurrentDeficit = MaxHealth - HealthAtCastTime
			If CurrentDeficit < 0 ;other fortify effects are messing up calculations
				CurrentDeficit = (HealthAtCastTime / PlayerRef.GetAVPercentage("Health")) - HealthAtCastTime
			EndIf
			int NewFortify = ((TotalHealing - CurrentDeficit) / 2) as int
			If SPEHardcoreMode.GetValue() == 1
				NewFortify /= 2 ;gain fewer hp/stamina in hardcore mode
			EndIf
			If NewFortify > FortifyAmount ;don't re-fortify to a lower number
				Game.AdvanceSkill("Restoration", (NewFortify - FortifyAmount) * 2) ;give xp for fortified health
				FortifyHealth(NewFortify)
			EndIf
			If PlayerRef.HasPerk(SPETemperance)
				FortifyStamina(NewFortify)
			EndIf
			CurrentHealth = PlayerRef.GetAV("Health")
			i = 0
		EndIf
	EndWhile
	;reset back to normal even if nothing happened
	SpellCast = NONE
	CurrentEffect = NONE
EndFunction

Event OnMagicEffectApply(ObjectReference akCaster, MagicEffect akEffect)
	If akCaster == PlayerRef && akEffect.HasKeyword(MagicRestoreHealth) && CurrentEffect == NONE ;don't overwrite mid-processing
		CurrentEffect = akEffect ;oddly, this processes before the spell cast.
	EndIf
EndEvent

Function FortifyHealth(int value)
	RemoveSpells() ;removing and re-adding spells makes this very accurate
	If PlayerRef.GetAVPercentage("Health") == 1
		MaxHealth = PlayerRef.GetAV("Health") as int + value
	Else
		MaxHealth -= FortifyAmount
		MaxHealth += value
	EndIf
	FortifyAmount = value
	;If value >= 256
	;	PlayerRef.AddSpell(SPEPerkFaithFort09, false)
	;	value -= 256
	;EndIf
	If value >= 128
		PlayerRef.AddSpell(SPEPerkFaithFort08, false)
		value -= 128
	EndIf
	If value >= 64
		PlayerRef.AddSpell(SPEPerkFaithFort07, false)
		value -= 64
	EndIf
	If value >= 32
		PlayerRef.AddSpell(SPEPerkFaithFort06, false)
		value -= 32
	EndIf
	If value >= 16
		PlayerRef.AddSpell(SPEPerkFaithFort05, false)
		value -= 16
	EndIf
	If value >= 8
		PlayerRef.AddSpell(SPEPerkFaithFort04, false)
		value -= 8
	EndIf
	If value >= 4
		PlayerRef.AddSpell(SPEPerkFaithFort03, false)
		value -= 4
	EndIf
	If value >= 2
		PlayerRef.AddSpell(SPEPerkFaithFort02, false)
		value -= 2
	EndIf
	If value >= 1
		PlayerRef.AddSpell(SPEPerkFaithFort01, false)
		value -= 1
	EndIf
EndFunction

Function FortifyStamina(int value)
	UpdatingStamina = true
	If FortifiedStamina <= value
		SPEPerkTemperanceShell.Cast(PlayerRef)
		FortifiedStamina = value
		;If value >= 256
		;	PlayerRef.AddSpell(SPEPerkTemperanceFort09, false)
		;	value -= 256
		;EndIf
		If value >= 128
			PlayerRef.AddSpell(SPEPerkTemperanceFort08, false)
			value -= 128
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort08)
		EndIf
		If value >= 64
			PlayerRef.AddSpell(SPEPerkTemperanceFort07, false)
			value -= 64
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort07)
		EndIf
		If value >= 32
			PlayerRef.AddSpell(SPEPerkTemperanceFort06, false)
			value -= 32
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort06)
		EndIf
		If value >= 16
			PlayerRef.AddSpell(SPEPerkTemperanceFort05, false)
			value -= 16
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort05)
		EndIf
		If value >= 8
			PlayerRef.AddSpell(SPEPerkTemperanceFort04, false)
			value -= 8
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort04)
		EndIf
		If value >= 4
			PlayerRef.AddSpell(SPEPerkTemperanceFort03, false)
			value -= 4
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort03)
		EndIf
		If value >= 2
			PlayerRef.AddSpell(SPEPerkTemperanceFort02, false)
			value -= 2
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort02)
		EndIf
		If value >= 1
			PlayerRef.AddSpell(SPEPerkTemperanceFort01, false)
			value -= 1
		Else
			PlayerRef.RemoveSpell(SPEPerkTemperanceFort01)
		EndIf
	ElseIf (value as float / FortifiedStamina as float) > 0.7
		SPEPerkTemperanceShell.Cast(PlayerRef) ;reset duration if the new fortify is at least sorta close to the old one
	EndIf
	UpdatingStamina = false
EndFunction

Function RemoveSpells()
	PlayerRef.RemoveSpell(SPEPerkFaithFort01)
	PlayerRef.RemoveSpell(SPEPerkFaithFort02)
	PlayerRef.RemoveSpell(SPEPerkFaithFort03)
	PlayerRef.RemoveSpell(SPEPerkFaithFort04)
	PlayerRef.RemoveSpell(SPEPerkFaithFort05)
	PlayerRef.RemoveSpell(SPEPerkFaithFort06)
	PlayerRef.RemoveSpell(SPEPerkFaithFort07)
	PlayerRef.RemoveSpell(SPEPerkFaithFort08)
	PlayerRef.RemoveSpell(SPEPerkFaithFort09)
EndFunction

Function RemoveStaminaSpells()
	If UpdatingStamina == false
		FortifiedStamina = 0
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort01)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort02)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort03)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort04)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort05)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort06)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort07)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort08)
		PlayerRef.RemoveSpell(SPEPerkTemperanceFort09)
	EndIf
EndFunction