Scriptname SPEGameSettingScript extends SPEPlayerOnlyAbility
{Modifies and updates game settings based on acquired perks.}

;Soul Scavenger
Perk Property SPESoulScavenger Auto
int LesserSoul
int CommonSoul
int GreaterSoul
int GrandSoul
bool SoulsModified = false

;Good Listener
GlobalVariable Property SPEGoodListenerCostMode Auto
Perk Property SPEGoodListener Auto
int TrainingAllowed
int Property TrainingMax = 99 Auto
int TrainingExpert
int TrainingMaster
int Property ExpertMult = 2 Auto
int Property MasterMult = 3 Auto
bool TrainingModified = false

;Rune Master
Perk Property SPERuneMaster Auto
int DefaultRunes
int Property ExtraRunes = 1 Auto
bool RunesModified = false

Function Initialize()
	RegisterForModEvent("SPE_UpdateGameSettings", "UpdateGameSettings")
	GetGameSettings() ;this should catch the default settings on first load
EndFunction

Event OnPlayerLoadGame()
	SoulsModified = false
	TrainingModified = false
	RunesModified = false
	GetGameSettings()
	SetGameSettings()
EndEvent

Function GetGameSettings()
	If SoulsModified == false
		LesserSoul = Game.GetGameSettingInt("iLesserSoulActorLevel")
		CommonSoul = Game.GetGameSettingInt("iCommonSoulActorLevel")
		GreaterSoul = Game.GetGameSettingInt("iGreaterSoulActorLevel")
		GrandSoul = Game.GetGameSettingInt("iGrandSoulActorLevel")
	EndIf
	If TrainingModified == false
		TrainingAllowed = Game.GetGameSettingInt("iTrainingNumAllowedPerLevel")
		TrainingExpert = Game.GetGameSettingInt("iTrainingExpertCost")
		TrainingMaster = Game.GetGameSettingInt("iTrainingMasterCost")
	EndIf
	If RunesModified == false
		DefaultRunes = Game.GetGameSettingInt("iMaxPlayerRunes")
	EndIf
EndFunction

Function UpdateGameSettings(string eventName, string strArg, float numArg, Form sender)
	SetGameSettings()
EndFunction

Function SetGameSettings()
	If PlayerRef.HasPerk(SPESoulScavenger)
		If LesserSoul == 0 ;somehow didn't get initialized
			GetGameSettings()
		EndIf
		SoulsModified = true
		Game.SetGameSettingInt("iLesserSoulActorLevel", 1)
		Game.SetGameSettingInt("iCommonSoulActorLevel", LesserSoul)
		Game.SetGameSettingInt("iGreaterSoulActorLevel", CommonSoul)
		Game.SetGameSettingInt("iGrandSoulActorLevel", GreaterSoul)
	ElseIf PlayerRef.HasPerk(SPESoulScavenger) == 0 && SoulsModified == true
		SoulsModified = false
		Game.SetGameSettingInt("iLesserSoulActorLevel", LesserSoul)
		Game.SetGameSettingInt("iCommonSoulActorLevel", CommonSoul)
		Game.SetGameSettingInt("iGreaterSoulActorLevel", GreaterSoul)
		Game.SetGameSettingInt("iGrandSoulActorLevel", GrandSoul)
	EndIf
	If PlayerRef.HasPerk(SPEGoodListener)
		If TrainingAllowed == 0 || TrainingExpert == 0 || TrainingMaster == 0
			GetGameSettings()
		EndIf
		TrainingModified = true
		If SPEGoodListenerCostMode.GetValue() == 0
			If Game.GetGameSettingInt("iTrainingExpertCost") != TrainingExpert && TrainingExpert != 0
				Game.SetGameSettingInt("iTrainingExpertCost", TrainingExpert)
				Game.SetGameSettingInt("iTrainingMasterCost", TrainingMaster)
			EndIf
			Game.SetGameSettingInt("iTrainingNumAllowedPerLevel", TrainingMax)
		Else
			If Game.GetGameSettingInt("iTrainingNumAllowedPerLevel") != TrainingAllowed && TrainingAllowed != 0
				Game.SetGameSettingInt("iTrainingNumAllowedPerLevel", TrainingAllowed)
			EndIf
			Game.SetGameSettingInt("iTrainingExpertCost", ExpertMult)
			Game.SetGameSettingInt("iTrainingMasterCost", MasterMult)
		EndIf
	ElseIf PlayerRef.HasPerk(SPEGoodListener) == 0 && TrainingModified == true
		TrainingModified = false
		Game.SetGameSettingInt("iTrainingNumAllowedPerLevel", TrainingAllowed)
		Game.SetGameSettingInt("iTrainingExpertCost", TrainingExpert)
		Game.SetGameSettingInt("iTrainingMasterCost", TrainingMaster)
	EndIf
	If PlayerRef.HasPerk(SPERuneMaster)
		If DefaultRunes == 0
			GetGameSettings()
		EndIf
		RunesModified = true
		Game.SetGameSettingInt("iMaxPlayerRunes", DefaultRunes + ExtraRunes)
	ElseIf PlayerRef.HasPerk(SPERuneMaster) == 0 && RunesModified == true
		RunesModified = false
		Game.SetGameSettingInt("iMaxPlayerRunes", DefaultRunes)
	EndIf
EndFunction
