Scriptname SPESEMainScript extends SPEPlayerOnlyAbility
{Manages the extra effects from the Side Effects perk.}

GlobalVariable Property GameDaysPassed Auto
Float Property Seconds = 0.0002315 AutoReadOnly
FormList Property SPESEAnytimeList Auto
Spell Property SPESECosmetic01 Auto
Spell[] CastMe
int SpellIndex = 0
bool ResetContext = false
bool CheckArmor = false
float ContextClearTime = 0.0

Function Initialize()
	RegisterForModEvent("SPE_SkillUp", "OnSkillUp")
	RegisterForModEvent("SPE_CombatStart", "OnCombatStart")
	RegisterForModEvent("SPE_CombatStop", "OnCombatStop")
	RegisterForModEvent("SPE_DiseaseStart", "OnDiseaseStart")
	RegisterForModEvent("SPE_DiseaseStop", "OnDiseaseStop")
	CastMe = new Spell[20]
	UpdateStatList()
EndFunction

;context effects
FormList Property SPESEContextList Auto

Spell Property SPESEAbsorbMagicka Auto
Spell Property SPESECureDisease Auto
Spell Property SPESEFortifyBarter Auto
Spell Property SPESEMakeEthereal Auto
Spell Property SPESEResistFire Auto
Spell Property SPESEResistFrost Auto
Spell Property SPESEResistMagic Auto
Spell Property SPESEResistShock Auto

Keyword Property LocTypeHabitationHasInn Auto
Keyword Property MagicDamageFire Auto
Keyword Property MagicDamageFrost Auto
Keyword Property MagicDamageShock Auto
Actor LastAttacker

Function AddToContextList(Spell SpellToAdd)
	If SPESEContextList.Find(SpellToAdd) == -1
		SPESEContextList.AddForm(SpellToAdd)
	EndIf
EndFunction

Event OnSkillUp(string eventName, string strArg, float numArg, Form sender)
	UpdateStatList()
EndEvent

Event OnCombatStart(string eventName, string strArg, float numArg, Form sender)
	AddToContextList(SPESEMakeEthereal)
EndEvent

Event OnCombatStop(string eventName, string strArg, float numArg, Form sender)
	SPESEContextList.RemoveAddedForm(SPESEMakeEthereal)
EndEvent

Event OnDiseaseStart(string eventName, string strArg, float numArg, Form sender)
	AddToContextList(SPESECureDisease)
	;Debug.Trace("Player has been diseased.")
EndEvent

Event OnDiseaseStop(string eventName, string strArg, float numArg, Form sender)
	SPESEContextList.RemoveAddedForm(SPESECureDisease)
	;Debug.Trace("Player is cured.")
EndEvent

Event OnLocationChange(Location akOldLoc, Location akNewLoc)
	If akNewLoc != NONE && akNewLoc.HasKeyword(LocTypeHabitationHasInn)
		AddToContextList(SPESEFortifyBarter)
	Else
		SPESEContextList.RemoveAddedForm(SPESEFortifyBarter)
	EndIf
EndEvent

Event OnHit(ObjectReference akAggressor, Form akSource, Projectile akProjectile, bool abPowerAttack, bool abSneakAttack, bool abBashAttack, bool abHitBlocked)
	Spell SpellHit = akSource as Spell
	If SpellHit
		If SpellHit.IsHostile()
			AddToContextList(SPESEResistMagic)
			AddToContextList(SPESEAbsorbMagicka)
			If SpellHit.HasKeyword(MagicDamageFire)
				AddToContextList(SPESEResistFire)
			ElseIf SpellHit.HasKeyword(MagicDamageFrost)
				AddToContextList(SPESEResistFrost)
			ElseIf SpellHit.HasKeyword(MagicDamageShock)
				AddToContextList(SPESEResistShock)
			EndIf
			ContextClearTime = GameDaysPassed.GetValue() + (6 * Seconds)
			RegisterForSingleUpdate(6)
		EndIf
	Else
		Actor Attacker = akAggressor as Actor
		If Attacker && Attacker.IsHostileToActor(PlayerRef) && Attacker.GetEquippedItemType(1) <= 7 ;melee hit
			LastAttacker = Attacker
			ContextClearTime = GameDaysPassed.GetValue() + (6 * Seconds)
			RegisterForSingleUpdate(6)
		EndIf
	EndIf
EndEvent

Event OnUpdate()
	SPESEContextTempList.Revert()
	While SpellIndex > 0
		SpellIndex -= 1
		CastMe[SpellIndex].Cast(PlayerRef)
		;CastMe[SpellIndex] = NONE
	EndWhile
	;Debug.Trace("Done Casting.")
	If ContextClearTime <= GameDaysPassed.GetValue()
		LastAttacker = NONE
		SPESEContextList.RemoveAddedForm(SPESEResistMagic)
		SPESEContextList.RemoveAddedForm(SPESEAbsorbMagicka)
		SPESEContextList.RemoveAddedForm(SPESEResistFire)
		SPESEContextList.RemoveAddedForm(SPESEResistFrost)
		SPESEContextList.RemoveAddedForm(SPESEResistShock)
		;Debug.Trace("Clearing context variables.")
	Else
		RegisterForSingleUpdate(1)
	EndIf
	If CheckArmor == true
		CheckArmor == false
		UpdateArmor()
	EndIf
EndEvent

;temp context effects
FormList Property SPESEContextTempList Auto

Spell Property SPESECurePoison Auto
Spell Property SPESEExplosion Auto
Spell Property SPESEFortifyHealRate Auto
Spell Property SPESEFortifyHealth Auto
Spell Property SPESEFortifyMagicka Auto
Spell Property SPESEFortifyMagickaRate Auto
Spell Property SPESEFortifyStamina Auto
Spell Property SPESEFortifyStaminaRate Auto
Spell Property SPESERestoreHealth Auto
Spell Property SPESERestoreMagicka Auto
Spell Property SPESERestoreStamina Auto

Keyword Property MagicAlchHarmful Auto
Keyword Property MagicAlchRestoreHealth Auto
Keyword Property MagicAlchFortifyHealth Auto
Keyword Property MagicAlchFortifyHealRate Auto
Keyword Property MagicAlchRestoreMagicka Auto
Keyword Property MagicAlchFortifyMagicka Auto
Keyword Property MagicAlchFortifyMagickaRate Auto
Keyword Property MagicAlchRestoreStamina Auto
Keyword Property MagicAlchFortifyStamina Auto
Keyword Property MagicAlchFortifyStaminaRate Auto

Event OnSpellCast(Form akSpell)
	Potion PotionDrank = akSpell as Potion
	If akSpell as Ingredient || (PotionDrank && PotionDrank.IsFood() == 0)
		If SpellIndex < 20 && Utility.RandomInt(1,3) == 3 ;one third chance
			int RandomEffect = Utility.RandomInt(1,100)
			;Debug.Trace("Casting Side Effect. Rolled a " + RandomEffect)
			If RandomEffect <= 30 ;context effects
				GoToState("Busy")
				;Debug.Trace("Processing context effect.")
				If SPESEContextTempList.GetSize() == 0 ;temp list empty, rebuild
					If PlayerRef.HasEffectKeyword(MagicAlchHarmful)
						SPESEContextTempList.AddForm(SPESECurePoison)
					EndIf
					If LastAttacker != NONE && LastAttacker.IsDead() == 0 && \
					LastAttacker.GetParentCell() == PlayerRef.GetParentCell() && PlayerRef.GetDistance(LastAttacker) <= 500
						SPESEContextTempList.AddForm(SPESEExplosion)
					EndIf
					If akSpell.HasKeyword(MagicAlchRestoreHealth) || akSpell.HasKeyword(MagicAlchFortifyHealth) || \
					akSpell.HasKeyword(MagicAlchFortifyHealRate)
						SPESEContextTempList.AddForm(SPESEFortifyHealRate)
						SPESEContextTempList.AddForm(SPESEFortifyHealth)
						If PlayerRef.GetAVPercentage("Health") < 0.9
							SPESEContextTempList.AddForm(SPESERestoreHealth)
						EndIf
					ElseIf akSpell.HasKeyword(MagicAlchRestoreMagicka) || akSpell.HasKeyword(MagicAlchFortifyMagicka) || \
					akSpell.HasKeyword(MagicAlchFortifyMagickaRate)
						SPESEContextTempList.AddForm(SPESEFortifyMagickaRate)
						SPESEContextTempList.AddForm(SPESEFortifyMagicka)
						If PlayerRef.GetAVPercentage("Magicka") < 0.9
							SPESEContextTempList.AddForm(SPESERestoreMagicka)
						EndIf
					ElseIf akSpell.HasKeyword(MagicAlchRestoreStamina) || akSpell.HasKeyword(MagicAlchFortifyStamina) || \
					akSpell.HasKeyword(MagicAlchFortifyStaminaRate)
						SPESEContextTempList.AddForm(SPESEFortifyStaminaRate)
						SPESEContextTempList.AddForm(SPESEFortifyStamina)
						If PlayerRef.GetAVPercentage("Stamina") < 0.9
							SPESEContextTempList.AddForm(SPESERestoreStamina)
						EndIf
					EndIf
				EndIf
				int ContextSize = SPESEContextList.GetSize()
				int TempSize = SPESEContextTempList.GetSize()
				int TotalSize = ContextSize + TempSize
				;Debug.Trace("Choosing from " + TotalSize + " effects.")
				If TotalSize > 0
					int index = Utility.RandomInt(0, TotalSize - 1)
					;Debug.Trace("Effect " + index + " chosen.")
					If index < ContextSize
						;Debug.Trace("Casting context effect.")
						;(SPESEContextList.GetAt(index) as Spell).Cast(PlayerRef)
						CastMe[SpellIndex] = SPESEContextList.GetAt(index) as Spell
					Else
						index -= ContextSize
						;Debug.Trace("Casting temp context effect.")
						;(SPESEContextTempList.GetAt(index) as Spell).Cast(PlayerRef)
						CastMe[SpellIndex] = SPESEContextTempList.GetAt(index) as Spell
					EndIf
					SpellIndex += 1
				EndIf
				;Debug.Trace("Done processing.")
				GoToState("")
			ElseIf RandomEffect <= 40 ;cosmetic effect
				;Debug.Trace("Choosing cosmetic effect.")
				;SPESECosmetic01.Cast(PlayerRef)
				CastMe[SpellIndex] = SPESECosmetic01
				SpellIndex += 1
			ElseIf RandomEffect <= 70 ;anytime effects
				int index = Utility.RandomInt(0, SPESEAnytimeList.GetSize() - 1)
				;Debug.Trace("Choosing anytime effect " + index + " of " + SPESEAnytimeList.GetSize())
				;(SPESEAnytimeList.GetAt(index) as Spell).Cast(PlayerRef)
				CastMe[SpellIndex] = SPESEAnytimeList.GetAt(index) as Spell
				SpellIndex += 1
			Else ;stat-based effects
				int index = Utility.RandomInt(0, SPESEStatList.GetSize() - 1)
				;Debug.Trace("Choosing stat effect " + index + " of " + SPESEStatList.GetSize())
				;SPESEStatList.GetAt(index) as Spell).Cast(PlayerRef)
				CastMe[SpellIndex] = SPESEStatList.GetAt(index) as Spell
				SpellIndex += 1
			EndIf
			RegisterForSingleUpdate(0)
		EndIf
	EndIf
EndEvent

State Busy
	Event OnSpellCast(Form akSpell)
	EndEvent
EndState

;stat-based effects
FormList Property SPESEStatList Auto

Spell Property SPESEFortifyAlteration Auto
Spell Property SPESEFortifyBlock Auto
Spell Property SPESEFortifyCarryWeight Auto
Spell Property SPESEFortifyConjuration Auto
Spell Property SPESEFortifyDestruction Auto
Spell Property SPESEFortifyFire Auto
Spell Property SPESEFortifyFrost Auto
Spell Property SPESEFortifyHeavyArmor Auto
Spell Property SPESEFortifyIllusion Auto
Spell Property SPESEFortifyLightArmor Auto
Spell Property SPESEFortifyMarksman Auto
Spell Property SPESEFortifyOneHanded Auto
Spell Property SPESEFortifyPickpocket Auto
Spell Property SPESEFortifyRestoration Auto
Spell Property SPESEFortifyShock Auto
Spell Property SPESEFortifySneak Auto
Spell Property SPESEFortifyTwoHanded Auto
Spell Property SPESEFrozenBlowsSpell Auto
Spell Property SPESEKeenEye Auto
Spell Property SPESELightningCloak Auto

Keyword Property ArmorShield Auto
Keyword Property ArmorHeavy Auto
Keyword Property ArmorLight Auto
Perk Property SPEFlameAffinity Auto
Perk Property SPEFrostAura Auto
Perk Property SPEFlowingCurrent Auto

Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	Armor EquippedArmor = akBaseObject as Armor
	If EquippedArmor
		CheckPiece(EquippedArmor)
	EndIf
EndEvent

Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
	Armor UnequippedArmor = akBaseObject as Armor
	If UnequippedArmor
		CheckPiece(UnequippedArmor)
	EndIf
EndEvent

Function CheckPiece(Armor ChangedPiece)
	If ChangedPiece.HasKeyword(ArmorShield) || ChangedPiece.HasKeyword(ArmorLight) || ChangedPiece.HasKeyword(ArmorHeavy)
		CheckArmor = true
		RegisterForSingleUpdate(0)
	EndIf
EndFunction

Function UpdateStatList() ;build a list of appropriate effects based on player stats
	GoToState("UpdatingStats")
	int Alteration = PlayerRef.GetBaseAV("Alteration") as int
	int Conjuration = PlayerRef.GetBaseAV("Conjuration") as int
	int Destruction = PlayerRef.GetBaseAV("Destruction") as int
	int Illusion = PlayerRef.GetBaseAV("Illusion") as int
	int Marksman = PlayerRef.GetBaseAV("Marksman") as int
	int OneHanded = PlayerRef.GetBaseAV("OneHanded") as int
	int Pickpocket = PlayerRef.GetBaseAV("Pickpocket") as int
	int Restoration = PlayerRef.GetBaseAV("Restoration") as int
	int Sneak = PlayerRef.GetBaseAV("Sneak") as int
	int TwoHanded = PlayerRef.GetBaseAV("TwoHanded") as int
	
	int HighestWeapon = Marksman
	If OneHanded > HighestWeapon
		HighestWeapon = OneHanded
	EndIf
	If TwoHanded > HighestWeapon
		HighestWeapon = TwoHanded
	EndIf
	If Pickpocket > HighestWeapon
		HighestWeapon = Pickpocket
	EndIf
	SPESEStatList.Revert() ;reset old list to make a new one
	If HighestWeapon >= Destruction
		SPESEStatList.AddForm(SPESEKeenEye)
		If HighestWeapon == Marksman
			SPESEStatList.AddForm(SPESEFortifyMarksman)
		EndIf
		If HighestWeapon == OneHanded
			SPESEStatList.AddForm(SPESEFortifyOneHanded)
		EndIf
		If HighestWeapon == TwoHanded
			SPESEStatList.AddForm(SPESEFortifyTwoHanded)
		EndIf
		If HighestWeapon == Pickpocket
			SPESEStatList.AddForm(SPESEFortifyPickpocket)
			SPESEStatList.AddForm(SPESEFrozenBlowsSpell)
		EndIf
	EndIf
	If Destruction >= HighestWeapon
		SPESEStatList.AddForm(SPESEFortifyDestruction)
		If PlayerRef.HasPerk(SPEFlameAffinity)
			SPESEStatList.AddForm(SPESEFortifyFire)
		EndIf
		If PlayerRef.HasPerk(SPEFrostAura)
			SPESEStatList.AddForm(SPESEFortifyFrost)
		EndIf
		If PlayerRef.HasPerk(SPEFlowingCurrent)
			SPESEStatList.AddForm(SPESEFortifyShock)
		EndIf
	EndIf
	If Sneak >= 35
		SPESEStatList.AddForm(SPESEFortifySneak)
	EndIf
	If Alteration >= 35
		SPESEStatList.AddForm(SPESEFortifyAlteration)
	EndIf
	If Conjuration >= 35
		SPESEStatList.AddForm(SPESEFortifyConjuration)
	EndIf
	If Illusion >= 35
		SPESEStatList.AddForm(SPESEFortifyIllusion)
	EndIf
	If Restoration >= 35
		SPESEStatList.AddForm(SPESEFortifyRestoration)
	EndIf
	UpdateArmor()
	;Debug.Trace("Stats updated. " + SPESEStatList.GetSize() + " possible effects.")
	GoToState("")
EndFunction

Function UpdateArmor()
	bool UsesBlock = (PlayerRef.GetAV("Block") >= 35 || PlayerRef.WornHasKeyword(ArmorShield))
	bool UsesLight = PlayerRef.WornHasKeyword(ArmorLight)
	bool UsesHeavy = PlayerRef.WornHasKeyword(ArmorHeavy)
	SPESEStatList.RemoveAddedForm(SPESELightningCloak)
	SPESEStatList.RemoveAddedForm(SPESEFortifyBlock)
	SPESEStatList.RemoveAddedForm(SPESEFortifyLightArmor)
	SPESEStatList.RemoveAddedForm(SPESEFortifyHeavyArmor)
	If UsesBlock || UsesLight || UsesHeavy
		SPESEStatList.AddForm(SPESELightningCloak)
	EndIf
	If UsesBlock
		SPESEStatList.AddForm(SPESEFortifyBlock)
	EndIf
	If UsesLight
		SPESEStatList.AddForm(SPESEFortifyLightArmor)
	EndIf
	If UsesHeavy
		SPESEStatList.AddForm(SPESEFortifyHeavyArmor)
	EndIf
EndFunction

State UpdatingStats
	Function UpdateStatList()
	EndFunction
EndState