Scriptname SPESkillUp extends Quest  
{Script to manage automatic addition of perks. Also imports existing characters.}

;autoperk lists
FormList Property SPEAutoAlchemy Auto
FormList Property SPEAutoAlteration Auto
FormList Property SPEAutoArchery Auto
FormList Property SPEAutoBlock Auto
FormList Property SPEAutoConjuration Auto
FormList Property SPEAutoDestruction Auto
FormList Property SPEAutoEnchanting Auto
FormList Property SPEAutoHeavyArmor Auto
FormList Property SPEAutoIllusion Auto
FormList Property SPEAutoLightArmor Auto
FormList Property SPEAutoLockpicking Auto
FormList Property SPEAutoOneHanded Auto
FormList Property SPEAutoPickpocket Auto
FormList Property SPEAutoRestoration Auto
FormList Property SPEAutoSmithing Auto
FormList Property SPEAutoSneak Auto
FormList Property SPEAutoSpeech Auto
FormList Property SPEAutoTwoHanded Auto

;autoperk variables
GlobalVariable Property AutoPerk Auto
GlobalVariable Property AutoPerkArchery Auto
GlobalVariable Property AutoPerkBlock Auto
GlobalVariable Property AutoPerkHeavyArmor Auto
GlobalVariable Property AutoPerkOneHanded Auto
GlobalVariable Property AutoPerkSmithing Auto
GlobalVariable Property AutoPerkTwoHanded Auto
GlobalVariable Property AutoPerkAlchemy Auto
GlobalVariable Property AutoPerkLightArmor Auto
GlobalVariable Property AutoPerkLockpicking Auto
GlobalVariable Property AutoPerkPickpocket Auto
GlobalVariable Property AutoPerkSneak Auto
GlobalVariable Property AutoPerkSpeech Auto
GlobalVariable Property AutoPerkAlteration Auto
GlobalVariable Property AutoPerkConjuration Auto
GlobalVariable Property AutoPerkDestruction Auto
GlobalVariable Property AutoPerkEnchanting Auto
GlobalVariable Property AutoPerkIllusion Auto
GlobalVariable Property AutoPerkRestoration Auto

;other lists and variables
GlobalVariable Property SPECheckAuto Auto
Perk Property RestorationDualCasting Auto
Perk Property ConjurationDualCasting Auto
Perk Property AlterationDualCasting Auto
Perk Property IllusionDualCasting Auto
Perk Property DestructionDualCasting Auto
Perk Property SPEDualCasting Auto
FormList Property SPEPerkListOld Auto ;list of old perks that changed to new ones
FormList Property SPEPerkListNew Auto ;the new ones, in the same order as the old
FormList Property SPEPerkListUnused Auto ;vanilla perks that are no longer used
FormList Property SPENoRefund  Auto ;no longer used
SPEUpdateScript Property SPEUpdateQuest Auto
Message Property SPENewPerk  Auto
Message Property SPENewPerks  Auto
Actor Property PlayerRef Auto
bool thorough = false

Event OnStoryIncreaseSkill (string asSkill)
	;trigger other scripts without having them also register in SM
	(self as Quest).SendModEvent("SPE_SkillUp", asSkill, PlayerRef.GetBaseAV(asSkill))
	Stop() ;stop quickly so it can catch another skill-up event
	AutoPerk() ;oddly enough, the script can run when the quest is stopped.
EndEvent

Function AutoPerk()
	If AutoPerk.GetValue() != 0 ;only add perks if autoperk is on.
		CheckAutoPerks()
	EndIf
EndFunction

Function CheckAutoPerks(bool silent = false)
	thorough = SPECheckAuto.GetValue() as bool
	int perksAdded = 0
	If AutoPerkAlchemy.GetValue() != 0
		perksAdded += AddPerks("Alchemy", SPEAutoAlchemy)
	EndIf
	If AutoPerkAlteration.GetValue() != 0
		perksAdded += AddPerks("Alteration", SPEAutoAlteration)
	EndIf
	If AutoPerkArchery.GetValue() != 0
		perksAdded += AddPerks("Marksman", SPEAutoArchery)
	EndIf
	If AutoPerkBlock.GetValue() != 0
		perksAdded += AddPerks("Block", SPEAutoBlock)
	EndIf
	If AutoPerkConjuration.GetValue() != 0
		perksAdded += AddPerks("Conjuration", SPEAutoConjuration)
	EndIf
	If AutoPerkDestruction.GetValue() != 0
		perksAdded += AddPerks("Destruction", SPEAutoDestruction)
	EndIf
	If AutoPerkEnchanting.GetValue() != 0
		perksAdded += AddPerks("Enchanting", SPEAutoEnchanting)
	EndIf
	If AutoPerkHeavyArmor.GetValue() != 0
		perksAdded += AddPerks("HeavyArmor", SPEAutoHeavyArmor)
	EndIf
	If AutoPerkIllusion.GetValue() != 0
		perksAdded += AddPerks("Illusion", SPEAutoIllusion)
	EndIf
	If AutoPerkLightArmor.GetValue() != 0
		perksAdded += AddPerks("LightArmor", SPEAutoLightArmor)
	EndIf
	If AutoPerkLockpicking.GetValue() != 0
		perksAdded += AddPerks("Lockpicking", SPEAutoLockpicking)
	EndIf
	If AutoPerkOneHanded.GetValue() != 0
		perksAdded += AddPerks("OneHanded", SPEAutoOneHanded)
	EndIf
	If AutoPerkPickpocket.GetValue() != 0
		perksAdded += AddPerks("Pickpocket", SPEAutoPickpocket)
	EndIf
	If AutoPerkRestoration.GetValue() != 0
		perksAdded += AddPerks("Restoration", SPEAutoRestoration)
	EndIf
	If AutoPerkSmithing.GetValue() != 0
		perksAdded += AddPerks("Smithing", SPEAutoSmithing)
	EndIf
	If AutoPerkSneak.GetValue() != 0
		perksAdded += AddPerks("Sneak", SPEAutoSneak)
	EndIf
	If AutoPerkSpeech.GetValue() != 0
		perksAdded += AddPerks("Speechcraft", SPEAutoSpeech)
	EndIf
	If AutoPerkTwoHanded.GetValue() != 0
		perksAdded += AddPerks("TwoHanded", SPEAutoTwoHanded)
	EndIf

	If !silent ;don't print a message in silent mode
		If perksAdded > 1
			SPENewPerks.show(perksAdded)
		ElseIf perksAdded == 1
			SPENewPerk.show()
		EndIf
	EndIf
	
	thorough = false ;check over, reset to normal
	SPECheckAuto.SetValue(0)
	
EndFunction

int Function AddPerks(string skillName, FormList autoPerks)
	int perksAdded = 0
	
	;first find the highest perk the player is eligible for
	int i = autoPerks.GetSize() - 1
	;If skillName == "Smithing"
	;	While (i*10+20) > PlayerRef.GetBaseAV(skillName)
	;		i -= 1
	;	EndWhile
	;Else
		While (i*25) > PlayerRef.GetBaseAV(skillName)
			i -= 1
		EndWhile
	;EndIf
	
	;now add perks until finding one the player already has
	While i > -1
		If !PlayerRef.HasPerk(autoPerks.GetAt(i) as Perk)
			PlayerRef.AddPerk(autoPerks.GetAt(i) as Perk)
			perksAdded += 1
			i -= 1
		ElseIf thorough ;keep checking if told to be thorough
			i -= 1
		Else ;we're not being thorough
			i = -1
		EndIf
	EndWhile

	return perksAdded
EndFunction

Function resetPerks(bool ModPerkPoints = true)
	int refundPenalty = 0
	int i = SPEPerkListOld.GetSize()
	If i != SPEPerkListNew.GetSize()
		Debug.Trace("SPERG Error: Cannot import character due to non-matching FormList sizes.")
		Debug.Notification("SPERG had an error importing your character.")
	Else
		While i > 0
			i -= 1
			If PlayerRef.HasPerk(SPEPerkListOld.GetAt(i) as Perk)
				PlayerRef.RemovePerk(SPEPerkListOld.GetAt(i) as Perk)
				PlayerRef.AddPerk(SPEPerkListNew.GetAt(i) as Perk)
				refundPenalty += 1
			EndIf
		EndWhile
		
		If PlayerRef.HasPerk(IllusionDualCasting) || PlayerRef.HasPerk(ConjurationDualCasting) || PlayerRef.HasPerk(AlterationDualCasting) || PlayerRef.HasPerk(RestorationDualCasting) || PlayerRef.HasPerk(DestructionDualCasting)
			PlayerRef.AddPerk(SPEDualCasting)
			refundPenalty += 1
		EndIf
	EndIf
	
	;get rid of vanilla perks
	i = SPEPerkListUnused.GetSize()
	While i > 0
		i -= 1
		PlayerRef.RemovePerk(SPEPerkListUnused.GetAt(i) as Perk)
	EndWhile
	
	If ModPerkPoints
		;set the perk points to where they should be, assuming SPERG was used from creation
		int perkPoints = Math.Floor((PlayerRef.GetLevel() - 1) / 2) - refundPenalty
		If perkPoints < 0
			perkPoints = 0
		EndIf
		Game.SetPerkPoints(perkPoints)
	EndIf

	thorough = true ;make sure all perks are checked
	CheckAutoPerks(true)

	SPEUpdateQuest.ImportCharacter()
	Stop()
EndFunction

Function NewCharacter()
	If AutoPerk.GetValue() != 0
		CheckAutoPerks(true) ;adds autoperks silently
	EndIf
	SPEUpdateQuest.NewCharacter() ;adds other necessary abilities for new characters
EndFunction