Scriptname SPESpellSwap Extends SPEPlayerOnlyAbility
{Changes out one set of spells for another, allowing perks to change spells completely.}

;This script functions off of parallel formlists. The following four lists must be ordered in the same way. The book in each position will teach the spell in the same position, it will be swapped for the book in the same position, and the new book will teach the new spell in the same position. If these do not match, the script will not work properly.

FormList Property BooksOld Auto ;list of spell books for the targeted spells
FormList Property BooksNew Auto ;parallel list of new spell books for the targeted spells
FormList Property SpellsOld Auto ;parallel list of old spells
FormList Property SpellsNew Auto ;parallel list of new spells

Function Initialize()
	SwapSpells()
EndFunction

Event OnEffectFinish(Actor akTarget, Actor akCaster)
	ResetSpells()
EndEvent

Function SwapSpells()
	int i = SpellsOld.GetSize()
	int books = 0
	While (i > 0) ;iterate through the spell list, replacing old spells/books with new
		i -= 1
		If PlayerRef.HasSpell(SpellsOld.GetAt(i) as Spell)
			PlayerRef.AddSpell(SpellsNew.GetAt(i) as Spell, false)
			If PlayerRef.GetEquippedSpell(0) == SpellsOld.GetAt(i) as Spell
				PlayerRef.EquipSpell(SpellsNew.GetAt(i) as Spell, 0)
			EndIf
			If PlayerRef.GetEquippedSpell(1) == SpellsOld.GetAt(i) as Spell
				PlayerRef.EquipSpell(SpellsNew.GetAt(i) as Spell, 1)
			EndIf
			PlayerRef.RemoveSpell(SpellsOld.GetAt(i) as Spell)
		EndIf
		;replace any spellbooks in inventory
		books = PlayerRef.GetItemCount(BooksOld.GetAt(i))
		If books
			PlayerRef.AddItem(BooksNew.GetAt(i), books, true)
			PlayerRef.RemoveItem(BooksOld.GetAt(i), books, true)
		EndIf
	EndWhile
	AddInventoryEventFilter(BooksOld)
	AddInventoryEventFilter(BooksNew)
EndFunction

Function ResetSpells()
	int i = SpellsNew.GetSize()
	int books = 0
	While (i > 0) ;iterate through the spell list, replacing old spells/books with new
		i -= 1
		If PlayerRef.HasSpell(SpellsNew.GetAt(i) as Spell)
			PlayerRef.AddSpell(SpellsOld.GetAt(i) as Spell, false)
			If PlayerRef.GetEquippedSpell(0) == SpellsNew.GetAt(i) as Spell
				PlayerRef.EquipSpell(SpellsOld.GetAt(i) as Spell, 0)
			EndIf
			If PlayerRef.GetEquippedSpell(1) == SpellsNew.GetAt(i) as Spell
				PlayerRef.EquipSpell(SpellsOld.GetAt(i) as Spell, 1)
			EndIf
			PlayerRef.RemoveSpell(SpellsNew.GetAt(i) as Spell)
		EndIf
		;replace any spellbooks in inventory
		books = PlayerRef.GetItemCount(BooksNew.GetAt(i))
		If books
			PlayerRef.AddItem(BooksOld.GetAt(i), books, true)
			PlayerRef.RemoveItem(BooksNew.GetAt(i), books, true)
		EndIf
	EndWhile
EndFunction

Event OnItemAdded(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akSourceContainer)
	int index = BooksOld.Find(akBaseItem)
	If index >= 0
		PlayerRef.RemoveItem(akBaseItem, aiItemCount, true)
		PlayerRef.AddItem(BooksNew.GetAt(index), aiItemCount, true)
	EndIf
EndEvent

Event OnItemRemoved(Form akBaseItem, int aiItemCount, ObjectReference akItemReference, ObjectReference akDestContainer)
	;probably dropped/stored/sold a new book - try to turn it back into an old book if possible
	If akItemReference || akDestContainer ;we can't (and probably don't need to) do anything if these don't exist
		int index = BooksNew.Find(akBaseItem)
		If index >= 0
			If akDestContainer ;item was sold/stored
				akDestContainer.RemoveItem(akBaseItem, aiItemCount, true)
				akDestContainer.AddItem(BooksOld.GetAt(index), aiItemCount, true)
			ElseIf aiItemCount == 1
				;item was dropped into the world. If multiple, spawning new ones can cause weirdness, so we're just going to leave that little loophole in. Congrats if you somehow found that loophole without reading this.
				akItemReference.PlaceAtMe(BooksOld.GetAt(index), 1)
				akItemReference.Disable()
				akItemReference.Delete()
			EndIf
		EndIf
	EndIf
EndEvent
