Scriptname SPESummonManagerScript extends Quest  
{Manages summoned creatures for the Phantom Beast, Phantom Armory, and Spirit Echo perks.}

Perk Property SPEPhantomBeast Auto
Perk Property SPEPhantomArmory Auto
Perk Property SPESpiritEcho Auto
Perk Property SPEPhantomFix Auto
Actor Property PlayerRef Auto

Spell Property SPEPhantomAxeAb Auto
Spell Property SPEPhantomSwordAb Auto
Spell Property SPEPhantomBowAb Auto
Spell Property SPEPhantomStaffAb Auto
Spell Property SPEPhantomKnightAb Auto

Spell Property SPEPerkSpiritEchoAb Auto

Spell Property SPEPhantomBearAb Auto
Spell Property SPEPhantomCatAb Auto
Spell Property SPEPhantomWolfAb Auto
Spell Property SPEPhantomChaurusAb Auto
Spell Property SPEPhantomSpiderAb Auto
Spell Property SPEPhantomSkeeverAb Auto
Spell Property SPEPhantomDeerAb Auto
Spell Property SPEPhantomChickenAb Auto
Spell Property SPEPhantomMammothAb Auto

GlobalVariable Property SPEBrawlMode Auto
SPEBrawlManager Property SPEBrawlQuest Auto
GlobalVariable Property SPESummonMode Auto

Spell beastSpell
Spell weaponSpell
Spell echoSpell

bool InCombat = false
bool readied = false
bool sneaking = false
bool BeastOnCooldown = false
bool keySummon = false

Event OnInit()
	RegisterForModEvent("SPE_CombatStart", "OnCombatStart")
	RegisterForModEvent("SPE_CombatStop", "OnCombatStop")
	RegisterForModEvent("SPE_SneakStart", "OnSneakStart")
	RegisterForModEvent("SPE_SneakStop", "OnSneakStop")
	RegisterForAnimationEvent(PlayerRef, "weaponDraw")
	RegisterForAnimationEvent(PlayerRef, "weaponSheathe")
EndEvent

Function OnCombatStart(string eventName, string strArg, float numArg, Form sender)
	InCombat = true
	If PlayerRef.HasPerk(SPEPhantomBeast) && !BeastOnCooldown
		PickBeast() ;chooses a random beast spell
	ElseIf beastSpell
		PlayerRef.RemoveSpell(beastSpell) ;make sure spell is gone before clearing
		beastSpell = NONE
	EndIf
	If PlayerRef.HasPerk(SPEPhantomArmory)
		PickWeapon() ;chooses a random weapon spell
	ElseIf weaponSpell
		PlayerRef.RemoveSpell(weaponSpell) ;make sure spell is gone before clearing
		weaponSpell = NONE
	EndIf
	If PlayerRef.HasPerk(SPESpiritEcho) && Utility.RandomInt(1,10) == 1
		echoSpell = SPEPerkSpiritEchoAb ;only one spell to pick, so pick it.
	ElseIf echoSpell
		PlayerRef.RemoveSpell(echoSpell) ;make sure spell is gone before clearing
		echoSpell = NONE
	EndIf
	AddSpells(quickSummon = false)
EndFunction

Function OnCombatStop(string eventName, string strArg, float numArg, Form sender)
	RemoveSpells(true)
	InCombat = false
EndFunction

Event OnAnimationEvent(ObjectReference akSource, string asEventName)
	If asEventName == "weaponDraw" && InCombat
		If SPESummonMode.GetValue() == 1 || (SPESummonMode.GetValue() == 2 && PlayerRef.IsSneaking())
			AddSpells()
		EndIf
	ElseIf asEventName == "weaponSheathe"
		If SPESummonMode.GetValue() == 1 || SPESummonMode.GetValue() == 2
			RemoveSpells(true)
		EndIf
	EndIf
EndEvent

Function OnSneakStop(string eventName, string strArg, float numArg, Form sender)
	If InCombat
		If SPESummonMode.GetValue() == 2 && PlayerRef.IsWeaponDrawn()
			AddSpells()
		EndIf
	EndIf
EndFunction

Function OnSneakStart(string eventName, string strArg, float numArg, Form sender)
	If SPESummonMode.GetValue() == 2
		RemoveSpells(true)
	EndIf
EndFunction

Function AddSpells(bool quickSummon = true)
	If InCombat
		If CanSummon(quickSummon)
			PlayerRef.AddPerk(SPEPhantomFix)
			RegisterForSingleUpdate(1)
			If beastSpell
				PlayerRef.AddSpell(beastSpell, false)
			EndIf
			If weaponSpell
				PlayerRef.AddSpell(weaponSpell, false)
			EndIf
			If echoSpell
				PlayerRef.AddSpell(echoSpell, false)
			EndIf
		EndIf
	EndIf
EndFunction

bool Function CreatureIsSummoned()
	If beastSpell && PlayerRef.HasSpell(beastSpell)
		return true
	ElseIf weaponSpell && PlayerRef.HasSpell(weaponSpell)
		return true
	ElseIf echoSpell && PlayerRef.HasSpell(echoSpell)
		return true
	Else
		return false
	EndIf
EndFunction

Event OnKeyDown(int KeyCode)
	If InCombat
		If CreatureIsSummoned()
			keySummon = false
			RemoveSpells(true)
		Else
			keySummon = true
			AddSpells()
		EndIf
	EndIf
EndEvent

Event OnUpdate()
	PlayerRef.RemovePerk(SPEPhantomFix)
EndEvent

Function RemoveSpells(bool forced = false)
	If forced || !CanSummon()
		If beastSpell
			PlayerRef.RemoveSpell(beastSpell)
		EndIf
		If weaponSpell
			PlayerRef.RemoveSpell(weaponSpell)
		EndIf
		If echoSpell
			PlayerRef.RemoveSpell(echoSpell)
		EndIf
		keySummon = false
	EndIf
EndFunction

Function BeastDied()
	PlayerRef.RemoveSpell(beastSpell)
	beastSpell = NONE
	BeastOnCooldown = true
	RegisterForSingleUpdateGameTime(3)
EndFunction

Function BeastUnsummoned()
	If CanSummon()
		PlayerRef.AddPerk(SPEPhantomFix)
		RegisterForSingleUpdate(1)
		PlayerRef.RemoveSpell(beastSpell)
		PlayerRef.AddSpell(beastSpell, false)
	EndIf
EndFunction

Function WeaponUnsummoned()
	If CanSummon()
		PlayerRef.AddPerk(SPEPhantomFix)
		RegisterForSingleUpdate(1)
		PlayerRef.RemoveSpell(weaponSpell)
		PlayerRef.AddSpell(weaponSpell, false)
	EndIf
EndFunction

Function EchoUnsummoned()
	If CanSummon()
		PlayerRef.AddPerk(SPEPhantomFix)
		RegisterForSingleUpdate(1)
		PlayerRef.RemoveSpell(echoSpell)
		PlayerRef.AddSpell(echoSpell, false)
	EndIf
EndFunction

Event OnUpdateGameTime()
	BeastOnCooldown = false
	If InCombat ;Initialize beast mid-combat if needed
		PickBeast()
		AddSpells(quickSummon = false)
	EndIf
EndEvent
	
Function PickBeast()
	int NumberOfTheBeast = Utility.RandomInt(1,50)
	If PlayerRef.IsInInterior()
		If NumberOfTheBeast <= 20
			beastSpell = SPEPhantomSkeeverAb
		ElseIf NumberOfTheBeast <= 35
			beastSpell = SPEPhantomSpiderAb
		ElseIf NumberOfTheBeast <= 50
			beastSpell = SPEPhantomChaurusAb
		EndIf
	Else
		If NumberOfTheBeast <= 15
			beastSpell = SPEPhantomWolfAb
		ElseIf NumberOfTheBeast <= 30
			beastSpell = SPEPhantomCatAb
		ElseIf NumberOfTheBeast <= 40
			beastSpell = SPEPhantomBearAb
		ElseIf NumberOfTheBeast <= 47
			beastSpell = SPEPhantomDeerAb
		ElseIf NumberOfTheBeast <= 49
			beastSpell = SPEPhantomChickenAb
		ElseIf NumberOfTheBeast == 50
			beastSpell = SPEPhantomMammothAb
		EndIf
	EndIf
EndFunction

Function PickWeapon()
	int PhantNum = Utility.RandomInt(1, 50)
	If PhantNum <= 15
		weaponSpell = SPEPhantomAxeAb
	ElseIf PhantNum <= 30
		weaponSpell = SPEPhantomSwordAb
	ElseIf PhantNum <= 45
		weaponSpell = SPEPhantomBowAb
	ElseIf PhantNum <= 48
		weaponSpell = SPEPhantomStaffAb
	ElseIf PhantNum <= 50
		weaponSpell = SPEPhantomKnightAb
	EndIf
EndFunction

bool Function CanSummon(bool quickSummon = false)
	If SPEBrawlMode.GetValue() == 1 || SPEBrawlQuest.CheckForBrawl() == 1 ;summon disabled during brawls
		return false
	ElseIf quickSummon ;forget the other checks and just summon
		return true
	ElseIf !InCombat ;can't summon out of combat
		return false
	ElseIf !PlayerRef.IsWeaponDrawn() && SPESummonMode.GetValue() != 3 ;must be readied unless manual mode is on
		return false
	ElseIf SPESummonMode.GetValue() == 2 && PlayerRef.IsSneaking() ;can't summon when sneaking with sneak mode on
		return false
	ElseIf SPESummonMode.GetValue() == 3 && keySummon == false ;can't summon during manual mode without hitting the key
		return false
	Else
		return true
	EndIf
EndFunction
