Scriptname SPEUnarmedManager extends SPEPlayerOnlyAbility
{Manages a fake weapon to allow unarmed to benefit from on-hit procs and melee haste.}

Weapon Property SPEUnarmedWeapon Auto
Weapon Property SPEUnarmedWeapon02 Auto
GlobalVariable Property SPEUnarmed Auto
GlobalVariable Property SPEUnarmedDamage Auto
GlobalVariable Property SPEBrawlMode Auto
GlobalVariable Property SPEHardcoreMode Auto
Keyword Property MagicAlchFortifyPickpocket Auto
Keyword Property MagicEnchFortifyPickpocket Auto
Spell Property SPEUnarmedFixAb01 Auto
Spell Property SPEUnarmedFixAb02 Auto
Spell Property SPEUnarmedFixAb03 Auto
Spell Property SPEUnarmedFixAb04 Auto
Spell Property SPEUnarmedFixAb05 Auto
Spell Property SPEUnarmedFixAb06 Auto
Spell Property SPEUnarmedFixAb07 Auto
Spell Property SPEUnarmedFixAb08 Auto
SPEBrawlManager Property SPEBrawlQuest Auto
float Property SkillMult = 0.4 Auto
float Property SkillMultHC = 0.2 Auto
bool CheckForUnarmed = false
int currentBonus = 0
Weapon UnarmedWeapon

Function Initialize()
	RegisterForModEvent("SPE_UpdateUnarmed", "UpdateUnarmed")
	RegisterForModEvent("SPE_SkillUp", "OnSkillUp")
	RegisterForModEvent("SPE_HardcoreModeOn", "OnHardcoreSwitch")
	RegisterForModEvent("SPE_HardcoreModeOff", "OnHardcoreSwitch")
EndFunction

Function OnSkillUp(string eventName, string strArg, float numArg, Form sender)
	If strArg == "Pickpocket"
		ManageBuffs()
	EndIf
EndFunction

Function OnHardcoreSwitch(string eventName, string strArg, float numArg, Form sender)
	ManageBuffs()
EndFunction

Function UpdateUnarmed(string eventName, string strArg, float numArg, Form sender)
	If strArg == "FullReset"
		RemoveUnarmed()
		RegisterForSingleUpdate(0)
	Else
		ManageBuffs()
	EndIf
EndFunction

Function ResetUnarmed(string eventName, string strArg, float numArg, Form sender)
EndFunction

Event OnObjectEquipped(Form akBaseObject, ObjectReference akReference)
	If akBaseObject as Weapon || akBaseObject as Armor
		RegisterForSingleUpdate(0)
	EndIf
EndEvent

Event OnObjectUnequipped(Form akBaseObject, ObjectReference akReference)
	If akBaseObject as Weapon || akBaseObject as Spell
		CheckForUnarmed = true
		RegisterForSingleUpdate(0)
	ElseIf akBaseObject as Armor
		RegisterForSingleUpdate(0)
	EndIf
EndEvent

Event OnPlayerLoadGame()
	CheckForUnarmed = true
	RegisterForSingleUpdate(0)
EndEvent

Event OnUpdate()
	If SPEBrawlMode.GetValue() == 1 ;just poll while manual brawl mode is on
		RegisterForSingleUpdate(1)
	Else
		ManageBuffs()
		If CheckForUnarmed && SPEBrawlQuest.CheckForBrawl() == 0
			CheckForUnarmed = false
			EquipUnarmed()
		EndIf
	EndIf
EndEvent

Event OnMagicEffectApply(ObjectReference akCaster, MagicEffect akEffect)
	If akEffect.HasKeyword(MagicAlchFortifyPickpocket) || akEffect.HasKeyword(MagicEnchFortifyPickpocket)
		ManageBuffs()
	EndIf
EndEvent

Function ManageBuffs()
	If SPEUnarmedDamage.GetValue() == 1 ;player wants buffs
		float CalculatedBonus = PlayerRef.GetAV("Pickpocket")
		If SPEHardcoreMode.GetValue() == 0
			CalculatedBonus *= SkillMult
		Else
			CalculatedBonus *= SkillMultHC
		EndIf
		CalculatedBonus *= ((PlayerRef.GetAV("PickpocketMod") / 100) + 1) ;enchantment bonus
		CalculatedBonus *= ((PlayerRef.GetAV("PickpocketPowerMod") / 100) + 1) ;alchemy bonus
		int bonus = CalculatedBonus as int ;quick way to truncate
		If currentBonus != bonus
			currentBonus = bonus
			RemoveBuffs()
			If bonus >= 128
				PlayerRef.AddSpell(SPEUnarmedFixAb08, false)
				bonus -= 128
			EndIf
			If bonus >= 64
				PlayerRef.AddSpell(SPEUnarmedFixAb07, false)
				bonus -= 64
			EndIf
			If bonus >= 32
				PlayerRef.AddSpell(SPEUnarmedFixAb06, false)
				bonus -= 32
			EndIf
			If bonus >= 16
				PlayerRef.AddSpell(SPEUnarmedFixAb05, false)
				bonus -= 16
			EndIf
			If bonus >= 8
				PlayerRef.AddSpell(SPEUnarmedFixAb04, false)
				bonus -= 8
			EndIf
			If bonus >= 4
				PlayerRef.AddSpell(SPEUnarmedFixAb03, false)
				bonus -= 4
			EndIf
			If bonus >= 2
				PlayerRef.AddSpell(SPEUnarmedFixAb02, false)
				bonus -= 2
			EndIf
			If bonus >= 1
				PlayerRef.AddSpell(SPEUnarmedFixAb01, false)
			EndIf
		EndIf
	Else ;player doesn't want buffs
		RemoveBuffs()
	EndIf
EndFunction

Function EquipUnarmed()
	
	If SPEUnarmed.GetValue() == 1 ;auto mode, equip invisible weapon
		UnarmedWeapon = SPEUnarmedWeapon
	Else ;hotkey/manual mode, equip visible weapon
		UnarmedWeapon = SPEUnarmedWeapon02
	EndIf
	
	;then make sure the player has the proper number of fists
	If PlayerRef.GetItemCount(UnarmedWeapon) > 2
		PlayerRef.RemoveItem(UnarmedWeapon, PlayerRef.GetItemCount(UnarmedWeapon) - 2, true)
	ElseIf PlayerRef.GetItemCount(UnarmedWeapon) < 2
		PlayerRef.AddItem(UnarmedWeapon, 2 - PlayerRef.GetItemCount(UnarmedWeapon), true)
	EndIf
	
	If SPEUnarmed.GetValue() <= 2 ;an auto-equip mode is active, so try equipping
		If PlayerRef.GetEquippedItemType(1) == 0 && PlayerRef.GetEquippedWeapon(false) != UnarmedWeapon
			If SKSE.GetVersion()
				PlayerRef.EquipItemEx(UnarmedWeapon, 1, false, true)
			Else
				PlayerRef.EquipItem(UnarmedWeapon, false, true)
			EndIf
		EndIf
		If SKSE.GetVersion() && PlayerRef.GetEquippedItemType(0) == 0 && PlayerRef.GetEquippedWeapon(true) != UnarmedWeapon
			PlayerRef.EquipItemEx(UnarmedWeapon, 2, false, true)
		EndIf
	EndIf
EndFunction

Function RemoveUnarmed()
	PlayerRef.RemoveItem(SPEUnarmedWeapon, PlayerRef.GetItemCount(SPEUnarmedWeapon), true)
	PlayerRef.RemoveItem(SPEUnarmedWeapon02, PlayerRef.GetItemCount(SPEUnarmedWeapon02), true)	
EndFunction

Function RemoveBuffs()
	PlayerRef.RemoveSpell(SPEUnarmedFixAb01)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb02)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb03)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb04)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb05)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb06)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb07)
	PlayerRef.RemoveSpell(SPEUnarmedFixAb08)
EndFunction
