Scriptname SPEUpdateScript extends Quest
{Takes care of any extra initialization stuff, particularly between versions.}

Quest Property SPEInitQuest Auto
SPEConfigMenuScript Property SPEConfigScript Auto
GlobalVariable Property SPEReinitializeMenu Auto
GlobalVariable Property SPEResetAliases Auto
GlobalVariable Property SPEDawnguardLoaded Auto
GlobalVariable Property SPEDragonbornLoaded Auto
Actor Property PlayerRef Auto
Float Property SPERGVersion Auto
Perk Property SPEBaseFixPerk Auto
Perk Property SPEBaseAbilitiesPerk Auto
Perk Property SPEBaseStaffManager Auto
Perk Property SPEBaseBoundSpells Auto
Perk Property SPEBaseWardSpells Auto

bool newCharacter = false
bool importCharacter = false

Event OnInit()
	If !SPEInitQuest.IsRunning()
		UpdateSPERG()
	EndIf
EndEvent

Function UpdateSPERG()
	float currentVersion = 3.90
	If SPEReinitializeMenu.GetValue() == 1
		SPEReinitializeMenu.SetValue(0)
		ResetAliases() ;in case the config alias slipped
		SPEConfigScript.Reinitialize()
	EndIf
	If SPEResetAliases.GetValue() == 1
		ResetAliases()
		SPEResetAliases.SetValue(0)
	EndIf
	If SPEUninstall.GetValue() ;user has requested uninstall, remove stuff
		int choice = SPEUninstallConfirm.Show()
		If choice == 0
			UninstallSPERG()
		Else
			SPEUninstall.SetValue(0)
		EndIf
	Else
		;check for DLC patches
		If SPEDawnguardLoaded.GetValue() == 1
			bool DawnguardLoaded = (Game.GetFormFromFile(0x0000A668, "SPERG-DG.esp") as bool)
			If DawnguardLoaded == false ;DLC patch was installed then removed.
				Debug.Trace("The Dawnguard DLC patch has been removed. This may cause some errors.")
				SPEDawnguardLoaded.SetValue(0)
			EndIf
		EndIf
		If SPEDragonbornLoaded.GetValue() == 1
			bool DragonbornLoaded = (Game.GetFormFromFile(0x00000800, "SPERG-DB.esp") as bool)
			If DragonbornLoaded == false
				SPEDragonbornLoaded.SetValue(0)
			EndIf
		EndIf
		
		If importCharacter ;character imports should display a finish message
			PlayerRef.AddPerk(SPEBaseFixPerk)
			PlayerRef.AddPerk(SPEBaseAbilitiesPerk)
			AddOptionalBasePerks()
			importCharacter = false
			SPEInitialized.Show()
		ElseIf newCharacter
			PlayerRef.AddPerk(SPEBaseFixPerk)
			PlayerRef.AddPerk(SPEBaseAbilitiesPerk)
			AddOptionalBasePerks()
			newCharacter = false ;new characters end silently
		Else ;updating characters need some extra handling
			If SPERGVersion < currentVersion
			EndIf
		EndIf
		SPERGVersion = currentVersion
	EndIf
EndFunction

Function AddOptionalBasePerks()
	PlayerRef.AddPerk(SPEBaseStaffManager)
	PlayerRef.AddPerk(SPEBaseBoundSpells)
	PlayerRef.AddPerk(SPEBaseWardSpells)
EndFunction

Function ImportCharacter()
	importCharacter = true
	UpdateSPERG()
EndFunction

Function NewCharacter()
	newCharacter = true
	UpdateSPERG()
EndFunction

Event OnUpdate()
	UpdateSPERG()
EndEvent

;alias stuff
ReferenceAlias Property ConfigAlias Auto
ReferenceAlias Property ImmunityAlias Auto
Perk Property SPEIronWill Auto

Function ResetAliases() ;sets all the proper aliases to the player in case they get removed or fail to update
	If ConfigAlias.GetRef() == NONE
		ConfigAlias.ForceRefTo(PlayerRef)
	EndIf
	If PlayerRef.HasPerk(SPEIronWill)
		ImmunityAlias.ForceRefTo(PlayerRef)
	Else
		ImmunityAlias.Clear()
	EndIf
EndFunction

;uninstall stuff
GlobalVariable Property SPEUninstall Auto
SPEBeginningQuestScript Property SPEBeginningQuest Auto
Quest Property SPEKeeningQuest Auto
Quest Property SPEQuestQuest Auto
Quest Property SPEConfigQuest Auto
Quest Property SPEPerksQuest Auto
Message Property SPEUninstallConfirm Auto
Message Property SPEUninstallStart Auto
Message Property SPEUninstallFinish Auto
Message Property SPEInitialized  Auto  

Function UninstallSPERG()
	SPEUninstallStart.Show()
	SPEBeginningQuest.ResetPerks()
	RemoveBasePerks()
	ConfigAlias.Clear()
	ImmunityAlias.Clear()
	SPEQuestQuest.Stop()
	SPEConfigQuest.Stop()
	SPEPerksQuest.Stop()
	If SPEKeeningQuest.IsRunning()
		SPEKeeningQuest.Stop()
	EndIf
	SPERGVersion = 0
	int choice = SPEUninstallFinish.Show()
	Stop()
EndFunction

Function RemoveBasePerks()
	PlayerRef.RemovePerk(SPEBaseBoundSpells)
	PlayerRef.RemovePerk(SPEBaseWardSpells)
	PlayerRef.RemovePerk(SPEBaseStaffManager)
	PlayerRef.RemovePerk(SPEBaseAbilitiesPerk)
	PlayerRef.RemovePerk(SPEBaseFixPerk)
EndFunction